package com.usky.gateway.service.mqtt.control;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.usky.gateway.service.config.mqtt.MqttOutConfig;
import com.usky.gateway.service.config.udp.UdpBaseConfig;
import com.usky.gateway.service.mqtt.MqttStrategy;
import com.usky.gateway.service.utils.ByteUtil;
import com.usky.gateway.service.utils.UdpUtil;
import com.usky.gateway.service.vo.MqttBaseVO;
import feign.form.util.CharsetUtil;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.nio.charset.Charset;
import java.util.Map;

@Service("control")
@Configuration
@Data
public class control implements MqttStrategy {
    @Value("${udp.deviceIp}")
    private String deviceIp;

    @Value("${udp.sendingPort}")
    private Integer sendingPort;

    @Autowired
    private UdpUtil udpUtil;
    @Autowired
    private ByteUtil byteUtil;
    @Resource
    private MqttOutConfig.MqttGateway mqttGateway;

    //处理下发命令消息，下发命令控制设备
    public String disposeMessage(MqttBaseVO mqttBaseVO) {
//        Map map_data = JsonUtils.fromJson(mqttBaseVO.getData().toString(), Map.class);
        JSONObject obj_data = JSONObject.parseObject(mqttBaseVO.getData().toString());
        Integer commandId = Integer.parseInt(obj_data.get("id").toString());
        Object params = JSONObject.toJSONString(obj_data.get("params"));
        JSONObject params_data = JSON.parseObject(params.toString());
        Integer deviceId = Integer.parseInt(params_data.getString("device_id"));

        //开门   17 40 00 00 E0 F4 4D 0D 01 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
        byte[] deviceByte = byteUtil.toLH(deviceId);
        byte[] requestBytes = new byte[] { 0x17, 0x40, 0x00, 0x00, deviceByte[0], deviceByte[1], deviceByte[2], deviceByte[3], 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
        byteUtil.printBytes(requestBytes);
        String responseData = udpUtil.sendUDPRequest(this.getDeviceIp(), this.getSendingPort(), requestBytes);

        //推送下发命令响应mqtt
        JSONObject jsonObject = new JSONObject();
        byte[] responseByte = responseData.getBytes(Charset.forName("ISO-8859-1"));  //"US-ASCII"
        byteUtil.printBytes(responseByte);

        if(responseByte[8] == 0x01){
            jsonObject.put("result","success");

        }else{
            JSONObject obj1 = new JSONObject();
            obj1.put("code",-1);
            obj1.put("message","open failed");
            jsonObject.put("error",obj1);

        }

        jsonObject.put("timeStamp",System.currentTimeMillis());
        jsonObject.put("id",commandId);

        String res_topic = mqttBaseVO.getTopic().replace("control","controlResponse");
        mqttGateway.sendToMqtt(res_topic,jsonObject.toJSONString());

        return null;
    }


}
