package com.usky.demo.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.usky.demo.domain.*;
import com.usky.demo.mapper.QueryInfluxdbDataMapper;
import com.usky.demo.service.DmpDeviceService;
import com.usky.demo.service.DmpProductService;
import com.usky.demo.service.QueryInfluxdbDataService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.demo.service.rocketmq.MyProducer;
import com.usky.demo.service.utils.TsdbUtils;
import com.usky.demo.service.vo.DeviceMapVO;
import com.usky.demo.service.vo.ProductMapVO;
import org.apache.commons.collections.ResettableListIterator;
import org.influxdb.dto.QueryResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2024-07-29
 */
@Service
public class QueryInfluxdbDataServiceImpl extends AbstractCrudService<QueryInfluxdbDataMapper, QueryInfluxdbData> implements QueryInfluxdbDataService {
    @Autowired
    private TsdbUtils tsdbUtils;
    @Autowired
    private DmpProductService dmpProductService;
    @Autowired
    private DmpDeviceService dmpDeviceService;

    @Value("${spring.influx.database}")
    private String database;


    @Cacheable(cacheNames = "productList",sync = true)
    public Map<String, ProductMapVO> getProductMap(){
        Map<String,ProductMapVO> productMap = new HashMap<>();
        LambdaQueryWrapper<DmpProduct> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DmpProduct::getDeleteFlag,0)
                .orderByDesc(DmpProduct::getId);
        List<DmpProduct> list = dmpProductService.list(queryWrapper);
        if(CollectionUtils.isNotEmpty(list)){
            for (int i = 0; i < list.size(); i++) {
                String productCode = list.get(i).getProductCode();
                ProductMapVO mapVO = new ProductMapVO();
                mapVO.setProductId(list.get(i).getId());
                mapVO.setProductCode(list.get(i).getProductCode());
                mapVO.setCreatedBy(list.get(i).getCreatedBy());
                mapVO.setTenantId(list.get(i).getTenantId());
                mapVO.setDeviceType(list.get(i).getDeviceType());

                productMap.put(productCode,mapVO);
            }
        }


        return productMap;
    }

    @Cacheable(cacheNames = "deviceList",sync = true)
    public Map<String, DeviceMapVO> getDeviceMap(){
        Map<String,DeviceMapVO> deviceMap = new HashMap<>();
        LambdaQueryWrapper<DmpDevice> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DmpDevice::getDeleteFlag,0)
                .orderByDesc(DmpDevice::getId);
        List<DmpDevice> list = dmpDeviceService.list(queryWrapper);
        if(CollectionUtils.isNotEmpty(list)){
            for (int i = 0; i < list.size(); i++) {
                String deviceId = list.get(i).getDeviceId();
                DeviceMapVO mapVO = new DeviceMapVO();
                mapVO.setProductCode(list.get(i).getProductCode());
                mapVO.setDeviceId(list.get(i).getDeviceId());
                mapVO.setDeviceUuid(list.get(i).getDeviceUuid());
                mapVO.setDeviceStatus(list.get(i).getServiceStatus());

                deviceMap.put(deviceId,mapVO);
            }
        }

        return deviceMap;
    }

    @Override
    public LastResultVO queryLastDeviceData(String deviceUUId){

        String tableName = deviceUUId;
        String query = "SELECT *::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
        List<Map<String, Object>> metrics = tsdbUtils.fetchRecords(query);
        String tagQuery = "SELECT \"deviceid\",*::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
        List<Map<String, String>> tag = tsdbUtils.fetchTagRecords(tagQuery);
        LastResultVO resultVO = new LastResultVO();
        if(tag.size() == 0){
            Map<String,String> tagMap = new HashMap<>();
            tagMap.put("deviceId","");
            tag.add(tagMap);
            resultVO.setDeviceUUId(deviceUUId);
            resultVO.setTags(tag);
            resultVO.setMetrics(metrics);
        }else{
            resultVO.setDeviceUUId(deviceUUId);
            resultVO.setTags(tag);
            resultVO.setMetrics(metrics);
        }

        return resultVO;
    }

    @Override
    public List<LastResultVO> queryLastDeviceData(LastRequestVO requestVO){
        List<LastResultVO> result = new ArrayList<>();
        List<String> deviceUUIds = requestVO.getDeviceUUId();
        if(CollectionUtils.isNotEmpty(deviceUUIds)){
            for (int i = 0; i < deviceUUIds.size(); i++) {
                String tableName = deviceUUIds.get(i);
                String query = "SELECT *::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
                List<Map<String, Object>> metrics = tsdbUtils.fetchRecords(query);
                String tagQuery = "SELECT \"deviceid\",*::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
                List<Map<String, String>> tag = tsdbUtils.fetchTagRecords(tagQuery);
                LastResultVO resultVO = new LastResultVO();
                resultVO.setDeviceUUId(deviceUUIds.get(i));
                resultVO.setTags(tag);
                resultVO.setMetrics(metrics);
                result.add(resultVO);
            }
        }

        return result;

    }

    @Override
    public HistoryResultVO queryHistoryDeviceData(String deviceUUId,String startTime,String endTime){
        List<MetricVO> metricList = new ArrayList<>();
        String tableName = deviceUUId;
        String query = "SELECT *::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
        QueryResult queryResult = tsdbUtils.query(query);
        String tagQuery = "SELECT \"deviceid\",*::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
        List<Map<String, String>> tag = tsdbUtils.fetchTagRecords(tagQuery);
        if(queryResult.getResults().get(0).getSeries() != null){
            List<String> fields = queryResult.getResults().get(0).getSeries().get(0).getColumns();
            if(CollectionUtils.isNotEmpty(fields)){
                String fieldQuery = "SELECT *::field FROM \""+tableName+"\" where time >= '"+startTime+"' and time <= '"+endTime+"' tz('Asia/Shanghai')";
                List<Map<String,Object>> metircs = tsdbUtils.fetchRecords(fieldQuery);

                if(CollectionUtils.isNotEmpty(metircs)){
                    for (int j = 0; j < fields.size(); j++) {
                        List<Map<String,Object>> metircItems = new ArrayList<>();
                        if(!"time".equals(fields.get(j))){
                            for (int k = 0; k < metircs.size(); k++) {
                                String field = fields.get(j);
                                Map<String,Object> map = new HashMap<>();
                                map.put("timestamp",metircs.get(k).get("time"));
                                map.put("value",metircs.get(k).get(field));
                                metircItems.add(map);
                            }
                            MetricVO metricVO = new MetricVO();
                            metricVO.setMetric(fields.get(j));
                            metricVO.setMetricItems(metircItems);
                            metricList.add(metricVO);
                        }

                    }
                }

            }
        }

        HistoryResultVO resultVO = new HistoryResultVO();
        resultVO.setDeviceUUId(deviceUUId);
        resultVO.setTags(tag);
        resultVO.setMetrics(metricList);
        return resultVO;
    }

    @Override
    public List<HistoryResultVO> queryHistoryDeviceData(HistoryRequestVO requestVO){
        List<HistoryResultVO> list = new ArrayList<>();
        List<String> deviceIds = requestVO.getDeviceUUId();
        String startTime = requestVO.getStartTime();
        String endTime = requestVO.getEndTime();
        for (int i = 0; i < deviceIds.size(); i++) {
            List<MetricVO> metricList = new ArrayList<>();
            String tableName = deviceIds.get(i);
            String query = "SELECT *::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
            QueryResult queryResult = tsdbUtils.query(query);
            String tagQuery = "SELECT \"deviceid\",*::field FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
            List<Map<String, String>> tag = tsdbUtils.fetchTagRecords(tagQuery);
            if(queryResult.getResults().get(0).getSeries() != null){
                List<String> fields = queryResult.getResults().get(0).getSeries().get(0).getColumns();
                if(CollectionUtils.isNotEmpty(fields)){
                    String fieldQuery = "SELECT *::field FROM \""+tableName+"\" where time >= '"+startTime+"' and time <= '"+endTime+"' tz('Asia/Shanghai')";
                    List<Map<String,Object>> metircs = tsdbUtils.fetchRecords(fieldQuery);

                    if(CollectionUtils.isNotEmpty(metircs)){
                        for (int j = 0; j < fields.size(); j++) {
                            List<Map<String,Object>> metircItems = new ArrayList<>();
                            if(!"time".equals(fields.get(j))){
                                for (int k = 0; k < metircs.size(); k++) {
                                    String field = fields.get(j);
                                    Map<String,Object> map = new HashMap<>();
                                    map.put("timestamp",metircs.get(k).get("time"));
                                    map.put("value",metircs.get(k).get(field));
                                    metircItems.add(map);
                                }
                                MetricVO metricVO = new MetricVO();
                                metricVO.setMetric(fields.get(j));
                                metricVO.setMetricItems(metircItems);
                                metricList.add(metricVO);
                            }

                        }
                    }

                }
            }

            HistoryResultVO resultVO = new HistoryResultVO();
            resultVO.setDeviceUUId(deviceIds.get(i));
            resultVO.setTags(tag);
            resultVO.setMetrics(metricList);
            list.add(resultVO);
        }

        return list;
    }

    @Override
    public List<LastInnerResultVO> last(LastInnerQueryVO requestVO){
        List<LastInnerResultVO> list = new ArrayList<>();
        List<String> metrics = requestVO.getMetrics();
        String tableName = requestVO.getDeviceUUId();
        for (int i = 0; i < metrics.size(); i++) {
            String query = "SELECT time,"+metrics.get(i)+" as attributeData FROM \""+tableName+"\" order by desc limit 1 tz('Asia/Shanghai')";
            List<Map<String, Object>> list1 = tsdbUtils.fetchRecords(query);
            if(CollectionUtils.isNotEmpty(list1)){
                LastInnerResultVO lastInnerResultVO = new LastInnerResultVO(metrics.get(i),Long.valueOf(list1.get(0).get("time").toString()),list1.get(0).get("attributeData"));
                list.add(lastInnerResultVO);
            }
        }

        return list;
    }

    @Override
    public HistorysInnerResultVO historyMetric(HistoryInnerRequestVO requestVO){
        String startTime = requestVO.getStartTime();
        String endTime = requestVO.getEndTime();
        String metric = requestVO.getMetric();
        String tableName = requestVO.getDeviceUUId();

        String query = "SELECT time,"+metric+" as attributeData FROM \""+tableName+"\" where time >='"+startTime+"' and time <= '"+endTime+"' tz('Asia/Shanghai')";
        List<Map<String, Object>> list1 = tsdbUtils.fetchRecords(query);
        List<MetricItemVO> metricItemVOList = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(list1)){
            for (int i = 0; i < list1.size(); i++) {
                MetricItemVO itemVO = new MetricItemVO(Long.valueOf(list1.get(i).get("time").toString()), list1.get(i).get("attributeData"));
                metricItemVOList.add(itemVO);
            }

        }
        HistorysInnerResultVO resultVO = new HistorysInnerResultVO(tableName,metric,metricItemVOList);

        return resultVO;
    }

    @Override
    public List<HistorysInnerResultVO> historyMetrics(HistorysInnerRequestVO requestVO){
        List<HistorysInnerResultVO> list = new ArrayList<>();
        String startTime = requestVO.getStartTime();
        String endTime = requestVO.getEndTime();
        List<String> metrics = requestVO.getMetrics();
        String tableName = requestVO.getDeviceUUId();

        for (int i = 0; i < metrics.size(); i++) {
            String metric = metrics.get(i);
            String query = "SELECT time,"+metric+" as attributeData FROM \""+tableName+"\" where time >='"+startTime+"' and time <= '"+endTime+"' tz('Asia/Shanghai')";
            List<Map<String, Object>> list1 = tsdbUtils.fetchRecords(query);
            List<MetricItemVO> metricItemVOList = new ArrayList<>();
            if(CollectionUtils.isNotEmpty(list1)){
                for (int j = 0; j < list1.size(); j++) {
                    MetricItemVO itemVO = new MetricItemVO(Long.valueOf(list1.get(j).get("time").toString()), list1.get(j).get("attributeData"));
                    metricItemVOList.add(itemVO);
                }

            }
            HistorysInnerResultVO resultVO = new HistorysInnerResultVO(tableName,metric,metricItemVOList);
            list.add(resultVO);
        }

        return list;
    }


}
